# encoding: utf-8

# ------------------------------------------------------------------------------
# Copyright (c) 2016 SUSE Linux GmbH, Nuernberg, Germany.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of version 2 of the GNU General Public License as published by the
# Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, contact SUSE Linux GmbH.
#
# ------------------------------------------------------------------------------
#
# Summary: SUSE Connect Program: product class
# Authors: Ilya Manyugin <ilya.manyugin@suse.com>

require 'json'
require 'erb'
require 'yast'
require 'base64'
require 'tmpdir'
require 'uri'

Yast.import 'UI'

include Yast::I18n
include ERB::Util

module SuseConnectProgram
  # Class containing information about the product and handling the JSON parsing
  class Product
    include Yast::Logger
    
    attr_reader :id, :type, :name, :tech_name, :repo_name, :version, :product_url,
                :description_short, :description_long, :product_info, :logo, :categories,
                :vendor_name, :vendor_url, :repository_url, :license_type, :license, :version,
                :doc_url, :keywords, :how_to_buy, :resolvable, :training_url, :eula_url,
                :repo_added, :installed, :dependencies, :json_s

    @@tmpdir_path = nil
    @@gui_product_template = nil
    @@ncurses_product_template = nil


    # Return all products defined in a file
    # @param file_path[String]
    def self.all(file_path)
      begin
        @@gui_product_template = File.read('/usr/share/YaST2/data/suse_connect_program/template_gui.erb')
        @@ncurses_product_template = File.read('/usr/share/YaST2/data/suse_connect_program/template_ncurses.erb')
      rescue Exception => e
        log.error "Could not read the ERB template files: #{e.message}"
        raise _('Could not read program data. Please reinstall the package.')
      end
      @@tmpdir_path = File.join(Dir.tmpdir, 'suse_connect_program')
      raise _('Could not create a temporary directory') unless FileUtils.mkdir_p @@tmpdir_path
      begin
        contents = File.read(file_path)
      rescue Exception => e
        log.error "Reading file '#{file_path}' failed: #{e.message}"
        raise _('Reading products file failed')
      end
      begin
        json = JSON.parse(contents, symbolize_names: true)
      rescue JSON::ParserError => e
        log.error "Error parsing json file '#{file_path}': #{e.message}"
        raise _('Error parsing the products file')
      end
      products = json.each_with_index.map do |entry, i|
        new(entry, i) if json_valid?(i, entry, file_path)
      end.compact
      products.sort_by { |p| [p.type, p.name.downcase] }
    end

    # Construct a Product instance from the provided parsed JSON hash and an ID
    # @param json [Hash]
    # @param id [Integer]
    def initialize(json, id)
      textdomain 'suse-connect-program'
      # the following fields must be in the JSON, otherwise the product won't be displayed
      @id = id
      @type = json[:product_type].to_sym
      @name = json[:product_name]
      @tech_name = @name.downcase.gsub(/[^[:word:]\s]/, '').gsub(/\s/, '_')
      @vendor_name = json[:vendor_name]
      @repo_name = @vendor_name.downcase.gsub(/[^[:word:]\s]/, '').gsub(/\s/, '_')
      @product_url = json[:product_url]
      @description_short = json[:product_description_short]
      @description_long = json[:product_description_long]
      @license_type = json[:license_type]
      # the following fields are optional
      @repository_url = json[:repository_url]
      if @repository_url
        # normalize the URL; the package manager always adds the trailing slash
        @repository_url += '/' unless @repository_url.end_with? '/'
      end
      @resolvable = json[:resolvable]
      @categories = json[:categories] ? json[:categories].join(', ') : ''
      if json[:product_logo] && !json[:product_logo].empty?
        @logo = File.join(@@tmpdir_path, @tech_name)
        begin
          File.open(@logo, 'wb') { |file| file.write(Base64.decode64(json[:product_logo])) }
        rescue SystemCallError
          @logo = nil
        end
      else
        @logo = nil
      end
      @license = json[:license]
      @eula_url = json[:eula_url]
      @version = json[:version]
      @doc_url = json[:documentation_url]
      @keywords = json[:keywords] ? json[:keywords].join(', ') : nil
      @training_url = json[:training_url]
      @how_to_buy = json[:how_to_buy]
      @vendor_url = nil
      @dependencies = json[:dependencies]
      @json_s = json
      update
    end

    # Return an array of all searchable fields' values of the product
    def searchable_fields
      [@name, @vendor_name, @description_long, @description_short, @categories, @keywords]
    end

    # Return an array of fields to be displayed in the table view
    def table_fields
      [@id, @name, @vendor_name, @description_short]
    end

    # Clean the temporary directory
    def self.clean_tmp
      FileUtils.rm_rf(@@tmpdir_path) if @@tmpdir_path
    end

    # Update the product information and regenerate the description
    # @param repo_added [Boolean]
    # @param installed [Boolead]
    def update(repo_added = false, installed = false)
      @repo_added = repo_added
      @installed = installed
      @product_info = generate_description
    end

    private

    # Check for mandatory keys in the JSON object
    # @param id [Integer] record ID in the file
    # @param json [Hash] parsed JSON
    # @param file_path [String] path to the products file (used only for logging)
    def self.json_valid?(id, json, file_path)
      unless json.key?(:product_type) &&
          ['rpmRepositoryProduct', 'webServiceProduct',
           'specialServiceProduct'].include?(json[:product_type])
        log.warn "Product #{id} is skipped in #{file_path}: unknown product_type."
        return false
      end
      required_fields = ['product_logo',
                         'product_description_short',
                         'product_description_long',
                         'product_url',
                         'product_type',
                         'vendor_name',
                         'product_name']
      if json['product_type'] == 'rpmRepositoryProduct'
        required_fields.concat(['resolvable', 'repository_url', 'eula_url'])
      end
      unless required_fields.all? { |field| json.key?(field.to_sym) }
        log.warn "Product #{id} is skipped in #{file_path}: one of the required fields "\
          "are not present."
        return false
      end
      # check the URLs
      unless json.all?{|k,v| k.to_s.include?('url') ? v =~ URI::regexp : true }
        log.warn "Product #{id} is skipped in #{file_path}: Incorrect data found in the URL fields."
        return false
      end
      true
    end

    # Generate an HTML description for the product
    def generate_description
      repo_added = @repo_added ? _('Added') : _('Not added')
      installed = @installed ? _('Installed') : _('Not installed')
      if Yast::UI.TextMode
        ERB.new(@@ncurses_product_template).result(binding)
      else
        ERB.new(@@gui_product_template).result(binding)
      end
    end
  end
end


  
